/* Tabela VENDITEMPED*/
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela VENDITEMPED*/
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela VENDITEMPED, utilizando a tabela Geral de Logs de Operaes */

IF EXISTS(SELECT *
          FROM SYSOBJECTS O, SYSUSERS U
          WHERE O.NAME = 'LOG_EXP_VENDITEMPED' AND
                TYPE   = 'TR'              AND
                U.UID  = O.UID             AND
                U.NAME = USER)

DROP TRIGGER LOG_EXP_VENDITEMPED
GO
CREATE TRIGGER LOG_EXP_VENDITEMPED
ON VENDITEMPED
FOR UPDATE,INSERT, DELETE
AS
BEGIN
  DECLARE @V_USUARIO             VARCHAR(40);
  DECLARE @V_TIPO                VARCHAR(1);
  DECLARE @V_TPMOVI              VARCHAR(1);
  DECLARE @V_ITPESQ              VARCHAR(40);
  DECLARE @V_DESC                VARCHAR(500);
  DECLARE @V_CDFILLOGOPER        VARCHAR(4);
  DECLARE @V_DSREMOTEADDR        VARCHAR(15);
  DECLARE @V_DSREMOTEHOST        VARCHAR(15);
  DECLARE @V_DSSERVERRDM         VARCHAR(20);
  DECLARE @V_DSSERVMETHOD        VARCHAR(30);
  DECLARE @V_DSREMOTEOPER        VARCHAR(12);
  DECLARE @V_DTOPER              DATETIME;

  DECLARE @V_CDFILIAL_NEW        VARCHAR(4);
  DECLARE @V_NRPEDIDO_NEW        VARCHAR(10);
  DECLARE @V_NRITEMPED_NEW       VARCHAR(10);
  DECLARE @V_CDVENDEDOR_NEW      VARCHAR(4);
  DECLARE @V_VRPECOMITPED_NEW    DOUBLE PRECISION;

  DECLARE @V_CDFILIAL_OLD        VARCHAR(4);
  DECLARE @V_NRPEDIDO_OLD        VARCHAR(10);
  DECLARE @V_NRITEMPED_OLD       VARCHAR(10);
  DECLARE @V_CDVENDEDOR_OLD      VARCHAR(4);
  DECLARE @V_VRPECOMITPED_OLD    DOUBLE PRECISION;


   SET @V_TIPO = '0';
   SET @V_DESC = '';
   SET @V_CDFILLOGOPER = '';
   SET @V_USUARIO = '-';

   DECLARE C_CURSOR_R CURSOR FOR
      SELECT
         'D' IDCOMANDO,
         NULL, NULL, NULL, NULL, NULL,
         CDFILIAL,
         NRPEDIDO,
         NRITEMPED,
         CDVENDEDOR,
         VRPECOMITPED
      FROM
         DELETED
      WHERE
         0 = (SELECT COUNT(*) FROM INSERTED)

      UNION ALL

      SELECT
         'U' IDCOMANDO,
         INSERTED.CDFILIAL,
         INSERTED.NRPEDIDO,
         INSERTED.NRITEMPED,
         INSERTED.CDVENDEDOR,
         INSERTED.VRPECOMITPED,

         DELETED.CDFILIAL,
         DELETED.NRPEDIDO,
         DELETED.NRITEMPED,
         DELETED.CDVENDEDOR,
         DELETED.VRPECOMITPED    

      FROM
         INSERTED,
         DELETED
      WHERE
         INSERTED.CDFILIAL   = DELETED.CDFILIAL  AND
         INSERTED.NRPEDIDO   = DELETED.NRPEDIDO  AND
         INSERTED.NRITEMPED  = DELETED.NRITEMPED AND
         INSERTED.CDVENDEDOR = DELETED.CDVENDEDOR

      UNION ALL

      SELECT
         'I' IDCOMANDO,
         INSERTED.CDFILIAL,
         INSERTED.NRPEDIDO,
         INSERTED.NRITEMPED,
         INSERTED.CDVENDEDOR,
         INSERTED.VRPECOMITPED,

         NULL, NULL, NULL, NULL, NULL
      FROM
         INSERTED
      WHERE
         0 = (SELECT COUNT(*) FROM DELETED)

   OPEN C_CURSOR_R;
   FETCH NEXT FROM C_CURSOR_R INTO @V_TIPO,
                                   @V_CDFILIAL_NEW, @V_NRPEDIDO_NEW, @V_NRITEMPED_NEW, @V_CDVENDEDOR_NEW, @V_VRPECOMITPED_NEW,

                                   @V_CDFILIAL_OLD, @V_NRPEDIDO_OLD, @V_NRITEMPED_OLD, @V_CDVENDEDOR_OLD, @V_VRPECOMITPED_OLD;

   WHILE (@@FETCH_STATUS = 0) BEGIN

      IF @V_TIPO = 'D' BEGIN

         IF @V_CDFILIAL_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Filial = ' + ISNULL(@V_CDFILIAL_OLD,'') + ' ';
         END;
         IF @V_NRPEDIDO_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Pedido = ' + ISNULL(@V_NRPEDIDO_OLD,'') + ' ';
         END;
         IF @V_NRITEMPED_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Item = ' + ISNULL(@V_NRITEMPED_OLD,'') + ' ';
         END;
         IF @V_CDVENDEDOR_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Vendedor = ' + ISNULL(@V_CDVENDEDOR_OLD,'') + ' ';
         END;
         IF @V_VRPECOMITPED_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Comisso = ' + ISNULL(CONVERT(VARCHAR,@V_VRPECOMITPED_OLD),'') + '% ';
         END;         

         SET @V_TPMOVI = '3';
         SET @V_ITPESQ = @V_CDFILIAL_OLD + @V_NRPEDIDO_OLD + @V_NRITEMPED_OLD + @V_CDVENDEDOR_OLD;
      END
      ELSE BEGIN
         IF @V_TIPO = 'U' BEGIN
             IF (@V_CDFILIAL_OLD     <> @V_CDFILIAL_NEW    ) OR
                (@V_NRPEDIDO_OLD     <> @V_NRPEDIDO_NEW    ) OR
                (@V_NRITEMPED_OLD    <> @V_NRITEMPED_NEW   ) OR
                (@V_CDVENDEDOR_OLD   <> @V_CDVENDEDOR_NEW  ) OR
                (@V_VRPECOMITPED_OLD <> @V_VRPECOMITPED_NEW) BEGIN

               SET @V_TPMOVI = '2';
               SET @V_ITPESQ = @V_CDFILIAL_OLD + @V_NRPEDIDO_OLD + @V_NRITEMPED_OLD + @V_CDVENDEDOR_OLD;
            END;
         END
         ELSE BEGIN
            SET @V_TPMOVI = '1';
            SET @V_ITPESQ = @V_CDFILIAL_NEW + @V_NRPEDIDO_NEW + @V_NRITEMPED_NEW + @V_CDVENDEDOR_NEW;
         END;

         IF ISNULL(@V_CDFILIAL_NEW, '!') <> ISNULL(@V_CDFILIAL_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Filial = ' + @V_CDFILIAL_NEW;
         END;
         IF ISNULL(@V_NRPEDIDO_NEW, '!') <> ISNULL(@V_NRPEDIDO_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Pedido = ' + @V_NRPEDIDO_NEW;
         END;
         IF ISNULL(@V_NRITEMPED_NEW, '!') <> ISNULL(@V_NRITEMPED_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Item = ' + @V_NRITEMPED_NEW;
         END;
         IF ISNULL(@V_CDVENDEDOR_NEW, '!') <> ISNULL(@V_CDVENDEDOR_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Vendedor = ' + @V_CDVENDEDOR_NEW;
         END;         
         IF ISNULL(CONVERT(VARCHAR,@V_VRPECOMITPED_NEW), '!') <> ISNULL(CONVERT(VARCHAR,@V_VRPECOMITPED_OLD), '!') BEGIN
            SET @V_DESC = @V_DESC + 'Comisso = ' + CONVERT(VARCHAR,@V_VRPECOMITPED_NEW);
         END;         
      END;

      IF @V_TPMOVI <> '0' BEGIN
         EXEC NMUSUARIO  @V_USUARIO;
         EXEC WEBTUNNING @V_DSREMOTEADDR,
                         @V_DSREMOTEHOST,
                         @V_DSSERVERRDM,
                         @V_DSSERVMETHOD,
                         @V_DSREMOTEOPER;
         IF NOT @V_DESC IS NULL BEGIN

            SET @V_DTOPER = GETDATE();

            INSERT INTO LOGOPERA
               (NMTABELA,
                DTOPERACAO,
                NMRESPOPER,
                IDTIPOOPER,
                CDPESQITEM,
                DSOPERACAO,
                CDFILLOGOPER,
                DSRDMUSUOPER,
                DSENDUSUOPER,
                DSNMUSUOPER,
                DSMETUSUOPER,
                CDORESPOPER)
            VALUES
               ('VENDITEMPED',
                @V_DTOPER,
                @V_USUARIO,
                @V_TPMOVI,
                @V_ITPESQ,
                SUBSTRING(@V_DESC,1,300),
                @V_CDFILLOGOPER,
                @V_DSSERVERRDM,
                @V_DSREMOTEADDR,
                @V_DSREMOTEHOST,
                @V_DSSERVMETHOD,
                @V_DSREMOTEOPER);

         END;
      END;
      FETCH NEXT FROM C_CURSOR_R INTO @V_TIPO,
                                      @V_CDFILIAL_NEW, @V_NRPEDIDO_NEW, @V_NRITEMPED_NEW, @V_CDVENDEDOR_NEW, @V_VRPECOMITPED_NEW,

                                      @V_CDFILIAL_OLD, @V_NRPEDIDO_OLD, @V_NRITEMPED_OLD, @V_CDVENDEDOR_OLD, @V_VRPECOMITPED_OLD;
   END;
   CLOSE C_CURSOR_R;
   DEALLOCATE C_CURSOR_R;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_EXP_VENDITEMPED','5.85.XXX';
GO