var engine = function(eventAggregator, templateManager, ScreenService, General, LibLoginController, ZhuAclController, LibUtilities, restEngine, ZHPromise, ApplicationContext, metaDataFactory, localStorageEngine, requestDataSourceEngine, ZhuAclConfig) {

    var aggregatorEvents = {
        BEFORE_PROJECT_STARTUP: "beforeProjectStartup",
        ON_CONTAINER_OPEN: "onContainerOpen",
        AUTH_SUCCESS: "authSuccess",
        LOGIN_SUCCESS: "loginSuccess",
        FAVORITES_ON_CHANGE: "ZhFavoritesOnChange"
    };
    var self = this;


    var regularExpressions = {
        LETTERS: /((a|A))/g,
        LETTERS_AND_NUMBERS: /((a|A|9))/g
    };

    var messages = {
        userDoesntHaveUnits: "Usuário não possui unidades relacionadas",
        unsetGeneralParams: "Parâmetros gerais não configurados",
        logout: "Deseja realmente sair do sistema?",
        withoutProfile: "Sem perfil selecionado",
        paramAccessControl: "Parametrização do Controle de Acesso por Perfil"
    };

    var messageTypes = {
        ERROR: "E",
        ALERT: "W"
    };

    var manContainers = {
        SELECIONAR_UNIDADE: "ger01742_selecionar_unidade",
        OPERADOR: "ger00002_operador"
    };
  var ON_UNIT_CHANGE = "onUnitChange";  
    var filial;

    // ************************** Projects Events ************************** // 
    eventAggregator.subscribe(aggregatorEvents.BEFORE_PROJECT_STARTUP, authUser);
    eventAggregator.subscribe(aggregatorEvents.AUTH_SUCCESS, onAuthSuccess);

    // ************************** Auth Methods ************************** //
    function authUser() {
        // Param that makes the product control the ACL Rules. 
        LibLoginController.authenticate()
            .then(validateLoginResponse)
            .catch(callbackError);
    }

    function onAuthSuccess(user) {
        fixFilterIssues();
        //check tipo de perfil
        setUserLocalVars(user);
        getParametersAccessControl().then(function(paramsAccessControl){
            setLocalVar('IDUTLPERFILFILIAL', paramsAccessControl.IDUTLPERFILFILIAL);
            checkGeneralParameters(user.nrorg).then(function() {
                if(paramsAccessControl.IDUTLPERFILFILIAL != 'N'){
                    setLocalVar('CDPERFILSELECIONADO', paramsAccessControl.CDPERFILSELECIONADO);
                        checkUnidadeDisp(user).then(function(filialDisp){
                            if (userHasSelectedUnit(user) && filialDisp && userHasSelectedProfile() ) {
                                setLocalProcessDate(user.cdfilialselecionada).then(function() {
                                    buildUserAndHeader();
                                    setLocalPurchasesCenter();
                                }, callbackError);
                            }
                            else {
                                tryToSetUserUnit(user);
                            }
                        });
                } else {
                    if (userHasSelectedUnit(user)) {
                        setLocalProcessDate(user.cdfilialselecionada).then(function() {
                            buildUserAndHeader();
                            setLocalPurchasesCenter();
                        }, callbackError);
                    }
                    else {
                        tryToSetUserUnit(user);
                    }
                }
            });
        }, callbackError);
    }

    var validateLoginResponse = function(response) {
        var defaultResponse = response;
        
        response = LibUtilities.changeLoginResponseToOldParams(response);
        
        if (!!response && !!response.cdoperador) {
            startSession(response).then(function(callback) {
                filial = Util.getNestedProperty(callback, ['dataset', 'filial', '0']);
                
                eventAggregator.publish(aggregatorEvents.LOGIN_SUCCESS, defaultResponse);
                eventAggregator.publish(aggregatorEvents.AUTH_SUCCESS, response);
            });
        }
    };

    // ************************** On Auth Success Methods ************************** //

    var startSession = function(user) {
        return General.callRestEngine("product#/startSession", "Row", user).then(null, callbackError);
    };

    var callbackError = function(err) {
        if (err.data && err.data.error) {
            ScreenService.showMessage(err.data.error, messageTypes.ERROR);
        } else {
            ScreenService.showMessage(JSON.stringify(err), messageTypes.ERROR);
        }
    };

    this.logout = function() {
        ScreenService.confirmMessage(messages.logout, messageTypes.ALERT).then(function() {
            removeLocalVars();
            LibLoginController.logout();
        });
    };

    var removeLocalVars = function() {
        localforage.clear();
        localStorage.clear();
    };

    // ************************** Unit Methods ************************** //
    var userHasSelectedUnit = function(user) {
        return !Util.isEmptyOrBlank(user.cdfilialselecionada);
    };

    var userHasSelectedProfile = function(){
            return (!Util.isEmptyOrBlank(getLocalVar('CDPERFILSELECIONADO')) || getLocalVar('IDUTLPERFILFILIAL') == 'P');
    };
    
    var tryToSetUserUnit = function(user) {
        var row = {
            'CDOPERADOR': user.cdoperador,
            'NRORG': getLocalVar('NRORG')
        };
        
        var currentScreenIsAccesProfile = window.location.href.includes('acl#access_profile');

        return General.callRestEngine("product#/setFilialSelecionadaOperador", "Row", row).then(function(response) {
            var data = response.dataset.filialSelecionada;
            var condRedirect = checkRedirecionamento();
            if ((!!data && !data['IDREDIRECT'] && condRedirect || currentScreenIsAccesProfile ) && getLocalVar('IDUTLPERFILFILIAL') !='S') {
                setUserUnit(user, data);
            }
            else {
                genericOpenWindow('man#' + manContainers.SELECIONAR_UNIDADE);
            }
        });
    };
    
    var checkRedirecionamento = function(){
        return (getLocalVar('IDUTLPERFILFILIAL') == 'S' && (!!getLocalVar('CDPERFILSELECIONADO') && getLocalVar('idutctrlacesweb') == 'S')) ;
    };

    var setUserUnit = function(user, data) {
        user.cdfilialselecionada = data['CDFILIALSELECIONADA'];
        user.cdfilial = data['CDFILIALSELECIONADA'];
        user.nmfilial = data['NMFILIAL'];

        // Updates Local Vars 
        setUserLocalVars(user);

        // Updates Lib Session
        General.callRestEngine("man#/lib_updateSession", 'FilterData', [{
            name: "OTHER_SESSION_VALUES",
            operator: "=",
            value: {
                'CDFILIAL': user.cdfilial,
                'NMFILIAL': user.nmfilial
            }
        }]).then(setLocalProcessDate(user.cdfilialselecionada).then(function() {
            buildUserAndHeader();
        }, callbackError));
    };

    var setUserLocalVars = function(user) {
        setLocalVar('LOGIN_CDOPERADOR', user.cdoperador);
        setLocalVar('CDOPERADOR', user.cdoperador);
        setLocalVar('NMOPERADOR', user.nmoperador);
        setLocalVar('CDFILIAL', user.cdfilialselecionada);
        setLocalVar('NMFILIAL', user.nmfilialselecionada);
        setLocalVar('NRORG', user.nrorg);
        setLocalVar('NRORGTRAB', user.nrorgtrab);
        setLocalVar('EMAIL', user.dsemailoper);
        setLocalVar('userId', user.cdoperador);
        setLocalVar('accessProfiles', user.perfilDeAcesso);
        setLocalVar('productIdLoginUrl', '1776');
    };

    // ************************** Util Methods ************************** //

    var replaceMask = function(value, regex) {
        return (value || '').toString().replace(regex, '*');
    };

    var checkGeneralParameters = function() {
        return getGeneralParameters().then(function(generalParameters) {
            if (!!generalParameters) {
                setGeneralParameters(generalParameters);
                if(getLocalVar('IDUTLPERFILFILIAL') != 'N'){
                    return getOperadorParameters().then(function(operatorParams){
                        setOperatorParameters(operatorParams);    
                    });
                }
            }
            else {
                ScreenService.showMessage(messages.unsetGeneralParams, messageTypes.ERROR);
            }
        });
    };

    var setGeneralParameters = function(generalParameters) {
        setLocalVar('CAD_CDPICTPRAT', replaceMask(generalParameters.CDPICTPRAT, regularExpressions.LETTERS));
        setLocalVar('GER_CDPICTPROD', replaceMask(generalParameters.CDPICTPROD, regularExpressions.LETTERS));
        setLocalVar('CDPICTPROD', replaceMask(generalParameters.CDPICTPROD, regularExpressions.LETTERS));
        setLocalVar('CDPICTCCTB', replaceMask(generalParameters.CDPICTCCTB, regularExpressions.LETTERS));
        setLocalVar('GER_CDPICTPRAT', replaceMask(generalParameters.CDPICTPRAT, regularExpressions.LETTERS_AND_NUMBERS));
        setLocalVar('CDPICTPRAT', replaceMask(generalParameters.CDPICTPRAT, regularExpressions.LETTERS_AND_NUMBERS));
    };
    
    var setOperatorParameters = function(operatorParameters) {
        setLocalVar('CDPERFILSELECIONADO', operatorParameters.CDPERFILACESSO);
        setLocalVar('NMPERFILSELECIONADO', operatorParameters.NMPERFILACESSO);
    };

    var getGeneralParameters = function() {
        return getTableByName("paramgeral").then(function(response) {
            return (!!response && !!response.length) ?
                response[0] :
                null;
        });
    };
    
    var getOperadorParameters = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
				"operator": "=",
				"value": getLocalVar('CDOPERADOR')  
            },
            {
                "name": "NRORG",
				"operator": "=",
				"value": getLocalVar('NRORG')  
            } 
        ];
        return General.callRestEngine("product#/paramOperador", "FilterData", filter).then(function(response) {
            return !!response.dataset['paramOperador'] ?
                response.dataset['paramOperador'] : [];
        });
    };

    var getTableByName = function(tableName, filter) {
        filter = filter || [];

        return General.callRestEngine("product#/crud/" + tableName, "FilterData", filter).then(function(response) {
            return !!response.dataset[tableName] ?
                response.dataset[tableName] : [];
        });
    };

    var setLocalProcessDate = function(unitSelected) {
        var filter = [{
            name: "CDFILIAL",
            operator: "=",
            value: unitSelected
        }];

        return getTableByName("paramfilial", filter).then(function(response) {
            var dtprocessa = moment().format("L");

            if (!!response.length && !!response[0].DTPROCESSA)
                dtprocessa = moment(response[0].DTPROCESSA, "DD/MM/YYYY hh:mm:ss").format("L");

            setLocalVar('DTPROCESSA', dtprocessa);

            // New event -> After all localvars are set
            eventAggregator.publish("onAllLocalVarsSetted");

            return dtprocessa;
        });
    };

    // ************************** User and Header methods ************************** //

    var userMethod = function() {
        genericOpenWindow('acl#my_profile');
    };

    var buildUserAndHeader = function() {
        var nmoperador = getLocalVar('NMOPERADOR');
        var dtprocessa = getLocalVar('DTPROCESSA');
        var cdfilial = getLocalVar('CDFILIAL');
        var nmfilial = getLocalVar('NMFILIAL');
        var perfil = getLocalVar('CDPERFILSELECIONADO') + ' - ' + getLocalVar('NMPERFILSELECIONADO');
        var unidade = cdfilial + " - " + nmfilial;
        var headerInfo = dtprocessa + " | " + unidade;
        if(getLocalVar('IDUTLPERFILFILIAL') != 'N'){
            
            if(getLocalVar('CDPERFILSELECIONADO') == null){
                 headerInfo = headerInfo + " | " + messages.withoutProfile;
            } else {
                headerInfo = headerInfo + " | " + perfil;
            }
            
            if(getLocalVar('IDUTLPERFILFILIAL') == 'P'){
                headerInfo = headerInfo + " | " + messages.paramAccessControl;
            }
        }
        ScreenService.buildUserData(nmoperador, [dtprocessa, unidade], userMethod);
        ScreenService.setHeaderInfo(headerInfo);
    };

    // ************************** Open Window Methods ************************** // 

    var requestModulesMetadata = function() {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function(response) {
            return response.data;
        });
    };

    var genericOpenWindow = function(windowName, newWindow) {
        requestModulesMetadata().then(function(modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (!!ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            }
            else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    this.openWindow = function(windowName, newWindow) {
        genericOpenWindow(windowName, newWindow);
    };
    
    var fixFilterIssues = function() {
        var userId = 'OM_' + localStorage.getItem('operador');
       
        if(ApplicationContext.ZhPreferences) {
            ApplicationContext.ZhPreferences.setUserID(userId);
        } else {
            ApplicationContext.zhCustomizationsService.registerUser(userId);
        }
    };
    
    var setLocalPurchasesCenter = function() {
		var cdfilcentcmp = '';

		if (filial && filial.CDFILCENTCMP) {
			cdfilcentcmp = filial.CDFILCENTCMP;
		} else {
			cdfilcentcmp = getLocalVar('CDFILIAL');
		}

		setLocalVar('CDFILCENTCMP', cdfilcentcmp);

		return cdfilcentcmp;
	};
	
	var checkUnidadeDisp = function(user){
    	var filter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value":user.nrorg
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": user.cdfilialselecionada
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": user.cdoperador
            },
            {
                "name": "CDPERFILSELECIONADO",
                "operator": "=",
                "value": user.nrperfilselecionado ? user.nrperfilselecionado : getLocalVar('CDPERFILSELECIONADO')
            }
        ];
    	
    	return restEngine.post('product#/checkFilialDisp', {
			"requestType": "FilterData",
			"filter": filter
        }).then(function(response) {
            return response.dataset.checkFilialDisp[0];
        }, function(error) {
            throw 'Ocorreu um erro!';
        });
    };
    
    var getParametersAccessControl = function(){
        return restEngine.post('product#/getParametersAccessControl', {
			"requestType": "FilterData",
			"filter": []
        }).then(function(response) {
            return response.dataset.getParametersAccessControl;
        }, function(error) {
            throw 'Ocorreu um erro!';
        });
        
    };
    
    this.selecionarUnidadeMenuOnclick = function(){
        if(getLocalVar('IDUTLPERFILFILIAL') != 'N'){
            self.openModalChangeAccessProfile();
        } else {
            self.openWindow('man#ger01742_selecionar_unidade');
        }
       
   };
    
    this.openModalChangeAccessProfile = function() {
        var widget = metaDataFactory.widgetFactory({
			"name": "changeProfileAndBranch",
			"showAsModal": true,
			"label": ScreenService.i18n("Trocar Perfil/Unidade"),
			"container": templateManager.container,
			"template": "widget/form.html",
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "NMPERFILACESSO",
					"label": ScreenService.i18n("Perfil"),
					"class": "6",
					"isVisible": true,
					"showSelectAs": "grid",
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"defaultSearchField": "ALL_FIELDS_SEARCH",
					"template": "field/select.html",
					"valueField": "NRPERFILACESSO",
					"descriptionField": "NMPERFILACESSO",
					"events": [
					    {
					       "name": "FieldOnChange",
					       "code": "engine.fieldNmPerfilAcessoOnChange(args);"
					    }
					],
					"hideEmptyValue": false,
					"dataSource": {
					    "name": "product#/perfisRelacionados",
                        "rest": true,
                        "localStorage": false,
                        "lazyLoad": false,
						"data":[]
					},
    				"dataSourceFilter": [
                        {
                            "name": "CDOPERADOR",
                            "operator": "=",
                            "value": "@getLocalVar('CDOPERADOR')@"
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('NRORG')@"
                        },
                        {
                            "name": "IDUTLPERFILFILIAL",
                            "operator": "=",
                            "value": "@getLocalVar('IDUTLPERFILFILIAL')@"
                        }
                    ],
                    "fields": [
                         {
                            "isVisible": true,
                            "showOnList": true,
                            "label": "Código",
                            "name": "NRPERFILACESSO",
                            "class": "12"
                        },
                        {
                            "isVisible": true,
                            "showOnList": true,
                            "label": "Nome",
                            "name": "NMPERFILACESSO",
                            "class": "12"
                        }
                    ],
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "NMFILIAL",
					"label": ScreenService.i18n("Unidade"),
					"class": "6",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"defaultSearchField": "ALL_FIELDS_SEARCH",
					"template": "field/select.html",
					"showSelectAs": "grid",
					"valueField": "CDFILIAL",
					"descriptionField": "NMFILIAL",
					"outData": {
					    "IDSTATUSBLOQ":"IDSTATUSBLOQ",
					    "IDBLOCKFILIAL": "IDBLOCKFILIAL",
					    "CDFILIAL":"CDFILIAL",
					    "NMFILIAL": "NMFILIAL"
					},
					"events": [
					    {
					       "name": "FieldBeforeSelectOpen",
					       "code": "engine.setProfileSelectInFilter(args)"
					    },
					    {
					       "name": "FieldOnChange",
					       "code": "engine.fieldOnChangeCheckFilialDisp(args)"
					    }
					],
					"hideEmptyValue": false,
					"dataSource": {
					    "name": "product#/unidadesRelacionadas",
                        "rest": true,
                        "localStorage": false,
                        "lazyLoad": false,
						"data":[]
					},
					"styles": [
                        {
                            "condition": "row.IDSTATUSBLOQ === 'S' && row.IDBLOCKFILIAL === 'S'",
                            "cssClass": "text-red"
                        }
                    ],
    				"dataSourceFilter": [
                        {
                            "name": "CDOPERADOR",
                            "operator": "=",
                            "value": "@getLocalVar('CDOPERADOR')@"
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('NRORG')@"
                        },
                        {
                            "name": "NRPERFILACESSO",
                            "operator": "=",
                            "value": ""
                        }
                    ],
                    "fields": [
                        {
                            "isVisible": true,
                            "showOnList": true,
                            "label": "Unidade",
                            "name": "CDFILIAL",
                            "class": "12"
                        },
                        {
                            "isVisible": true,
                            "showOnList": true,
                            "label": "Nome",
                            "name": "NMFILIAL",
                            "class": "12"
                        }
                    ],
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"hideIcon": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 1,
					"name": "tranferFilter",
					"label": ScreenService.i18n("Aplicar"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "engine.applyChangeProfile(args.owner.widget)"
						}
					]
				}
			]
        });
        widget.getField('NMPERFILACESSO').reload().then(function(data){
            var retorno = data.dataset.getProfiles.filter(function(profile){
                return profile.NRPERFILACESSO == getLocalVar('CDPERFILSELECIONADO');
            });
            if(retorno.length > 0){
                widget.currentRow.NRPERFILACESSO = retorno[0].NRPERFILACESSO;
                widget.currentRow.NMPERFILACESSO = retorno[0].NMPERFILACESSO;
            }
            widget.getField('NMFILIAL').dataSourceFilter[2].value = retorno[0].NRPERFILACESSO;
            widget.getField('NMFILIAL').reload().then(function(data){
                var retornoFilial = data.dataset.getBranches.filter(function(branch){
                    if(data.dataset.getBranches.length > 1){
                        return branch.CDFILIAL != getLocalVar('CDFILIAL');
                    } else {
                        return branch.CDFILIAL == getLocalVar('CDFILIAL');
                    }
                });
                widget.currentRow.CDFILIAL = retornoFilial[0].CDFILIAL;
                widget.currentRow.NMFILIAL = retornoFilial[0].NMFILIAL;
            });
        });
        ScreenService.openPopup(widget);
    };
    
    this.setProfileSelectInFilter = function(args){
        var widget = args.owner.owner.widget;
        if(widget != undefined){
            var NRPERFILACESSO = widget.currentRow.NRPERFILACESSO;
            var fieldBranch = widget.getField('NMFILIAL');
            fieldBranch.dataSourceFilter[2].value = NRPERFILACESSO;
            fieldBranch.reload();
        }
    };
    
    this.applyChangeProfile = function(widget){
        
        var mensagem = "Deseja confirmar a troca de unidade ? ";
        if(widget.isValid()){
            var row = {
                'nrorg': getLocalVar('NRORG'),
                'cdoperador': getLocalVar('CDOPERADOR'),
                'nrperfilacesso': widget.currentRow.NRPERFILACESSO,
                'cdfilial': widget.currentRow.CDFILIAL,
                'nmfilial': widget.currentRow.NMFILIAL
            };
            checkUnidadeDisp(row).then(function(flagUnidadeDisp){
    			if(!flagUnidadeDisp){
    				ScreenService.showMessage('Unidade Indisponivel');
    			} else {
    				ScreenService.confirmMessage(mensagem, "question", changeUnit.bind(row));
    			}
    		});
        }
    };
    
    var changeUnit = function () {
		var NRORG			= getLocalVar('NRORG'), 
		    CDFILIAL		= this.cdfilial, 
			NMFILIAL		= this.nmfilial, 
			PERFILACESSO    = this.nrperfilacesso, 
			CDOPERADOR		= getLocalVar("CDOPERADOR");
		
	self.alterarEstrutura(NRORG, CDFILIAL, NMFILIAL, CDOPERADOR, PERFILACESSO)
			.then(function(response) {
				var updateSessionData = {
			    	"CDFILIAL"          : CDFILIAL,
			    	"NMFILIAL"          : NMFILIAL,
			    	"ACL_ADMINISTRATOR" : response.dataset.operador[0].ACL_ADMINISTRATOR
			    };
			    
			    if(PERFILACESSO == null || PERFILACESSO == "" || PERFILACESSO == undefined) {
			        perfil = null;
			    } else {
			        var perfil = [
                        {
                            ID: PERFILACESSO,
                            IDPERFILACESSOADM: 'N'
                        }
                    ];    
			    }
				
				LibLoginController.updateSession(null, perfil, updateSessionData)
					.then(function () {
						updateLocalSession(CDFILIAL, NMFILIAL);
					})
					.catch(callbackError);
			});
	};
	
	this.alterarEstrutura = function (NRORG, CDFILIAL, NMFILIAL, CDOPERADOR, PERFIL) {
        
        var filter = {
            'CDFILIAL'    : CDFILIAL, 
            'CDOPERADOR'  : CDOPERADOR,
            'PERFIL'	  : PERFIL,
            'NRORG'       : NRORG          
        };
        
        return General.callRestEngine('product#/alterarEstrutura', 'Row', filter);
    };
    
    var updateLocalSession = function(CDFILIAL, NMFILIAL) {
		retornaDataProcessamento(getLocalVar('CDFILIAL'), getLocalVar('NRORG')).then(function (DTPROCESSA) {
			
			var nmoperador = getLocalVar("NMOPERADOR");
			var unitDescription = CDFILIAL + ' - ' + NMFILIAL;
			var headerInfo = DTPROCESSA + " | " + unitDescription;
			
			buildUserAndHeader(nmoperador, [DTPROCESSA, unitDescription], headerInfo);
			
			var data = {
				'DTPROCESSA' : DTPROCESSA,
				'CDFILIAL'	 : CDFILIAL,
				'NMFILIAL'   : NMFILIAL
			};
			
			setLocalVars(data);
			eventAggregator.publish(ON_UNIT_CHANGE, data);
			window.location.reload();
			
		}, callbackError);
	};
    
    
    var retornaDataProcessamento = function(cdfilial, nrorg) {
		return self.getParamfilial(nrorg, cdfilial).then(function(callback) {
			if(!!callback.dataset.paramfilial && callback.dataset.paramfilial.length > 0 ) {
				var dtprocessa = callback.dataset.paramfilial[0].DTPROCESSA;
	
				if(!!dtprocessa)
					return moment(dtprocessa,'DD/MM/YYYY HH:mm:ss').format('L');
			}
			
			return moment().format('L');
		}).catch(callbackError);
	};
	
	this.getParamfilial = function(nrorg, cdfilial) {
		nrorg    = nrorg    || getLocalVar("NRORG") || 'NRORG_LOGADO';
		cdfilial = cdfilial || getLocalVar("CDFILIAL") || 'FILIAL_LOGADA';
		
		var filter = [
		    {
			    "name" :"NRORG",
			    "operator": "=",
			    "value": nrorg
		    },
		    {
			    "name": "CDFILIAL",
			    "operator": "=",
			    "value" : cdfilial
		    }
		];
		
		return General.callRestEngine('man#/crud/paramfilial', 'FilterData', filter);
	};
	
	var setLocalVars = function (unidade) {
		setLocalVar("CDFILIAL",   unidade.CDFILIAL);
		setLocalVar("NMFILIAL",   unidade.NMFILIAL);
		setLocalVar("DTPROCESSA", unidade.DTPROCESSA);
	};
	
	this.fieldNmPerfilAcessoOnChange = function(args){
	    var widget = args.owner.owner.widget;
	    var fieldFilial = widget.getField('NMFILIAL');
	    fieldFilial.dataSourceFilter[2].value = widget.currentRow.NRPERFILACESSO;
	    fieldFilial.reload().then(function(){
	       if(fieldFilial.dataSource.data.length > 0 && !!widget.currentRow.NMPERFILACESSO){
	            widget.currentRow.CDFILIAL = fieldFilial.dataSource.data[0].CDFILIAL;
    	        widget.currentRow.NMFILIAL = fieldFilial.dataSource.data[0].NMFILIAL;
	       } else {
        	    widget.currentRow.CDFILIAL = null;
        	    widget.currentRow.NMFILIAL = null;
	           
	       }
	    });
	};
	
	this.fieldOnChangeCheckFilialDisp = function(args){
	    var widget = args.owner.owner.widget;
	    if(widget.currentRow.IDBLOCKFILIAL == 'S' &&  widget.currentRow.IDSTATUSBLOQ == 'S'){
	        widget.currentRow.CDFILIAL = null;
	        widget.currentRow.NMFILIAL = null;
	        ScreenService.showMessage('Unidade Bloqueada para este Perfil! Entre em contato com a franqueadora!');
	    }
	}; 
	
	
    
    eventAggregator.subscribe(aggregatorEvents.ON_CONTAINER_OPEN, menuFavoritos.bind(this));
    
    /**
     * Teste Menu Favoritos
     */
    function menuFavoritos() {
        
        // Aguarda renderização do componente de Acesso Rápido
        setTimeout(function() {
            
            var groupId   = localStorageEngine.getLocalVar('accessProfiles');
            var projectId = localStorageEngine.getLocalVar('productIdLoginUrl');
            
            if(projectId && groupId){
                // Busca o Controle de Acesso do Operador Logado
                requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', ZhuAclConfig.getDefaultDataSourceFilter(groupId, projectId), 1, undefined, undefined, undefined, 120000).then(function(response){
        			var dataset = response.dataset || {};
        			
        			if(dataset){
        			    
        			    requestDataSourceEngine.getDataSource('product#/lib_findFavorites', [], 1, undefined, undefined, undefined, 120000).then(function(result) {
                           
                            var favorites = result.dataset.favorites;
                            
                            favorites = favorites[0].map(item => {
                              return {
                                id: item.ID,
                                name: item.NAME,
                                label: item.LABEL
                              };
                            });
            			    
            			    if(favorites){
                			    
                			    // Verifica Controle de Acesso do Menu Favoritos
                			    verificaAcessoFavoritos(dataset, favorites).then(function(favorites){
                                    
                                    // Inclui Favoritos no componente de Acesso Rápidp
                                    favorites = JSON.parse(JSON.stringify(favorites));
                	                eventAggregator.publish('ZhFavoritesOnLoad', favorites);
                			    }.bind(this));    
        			    
            			    }
                           
                        }, function(error) {});
        			}
        		});
            }
        }, 2000);
    };
    
    function verificaAcessoFavoritos(dataset, favorites){
        var deferred = ZHPromise.defer();
        
        // Variavel de Posição no Array
        var pos = null;
        
        // Captura ID's dos Menus Favoritos
        var menuFavoritesIds = favorites.map(function(a) {return a.id;});
        
        // Regra Padrão
	    var defaultRule = dataset.zhu_access_control_default_rule.defaultRule;
        
        // Preenche Regra Padrão para todos os Favoritos
        favorites.forEach(function(favorite){
	        // Teste de Inativação
	        //favorite.rule = 'I';
	        
	        favorite.rule = defaultRule;
	    });
        
        // Percorre o Controle de Acesso dos Menus
        dataset.zhu_access_control_rule_login.forEach(function(acl){
            // Identifica existência de Controle de Acesso para o Menu
	        if(menuFavoritesIds.includes(acl.UNIQUE_ID)){
	            pos = menuFavoritesIds.indexOf(acl.UNIQUE_ID);
	            // Preenche Regra para o Menu
	            favorites[pos].rule = acl.RULE;
	        }
	    });
	    
	    // Permanece somente com os Favoritos ATIVOS(A)
	    favorites = favorites.filter(function(favorite) {
	        return ((favorite.rule == 'A') || (favorite.rule == 'R'));
	    });
        
        deferred.resolve(favorites);
        
        return deferred.promise;
    };
    
    eventAggregator.subscribe(aggregatorEvents.FAVORITES_ON_CHANGE, (args) => {
        // Adiciona/Deleta Menu Favoritos
        // args.fullList = array<{ id, name, label }>
        // args.changed = { item: { id, name, label }, status: 'add'|'delete' }
        General.callRestEngine('product#/lib_changeFavorites', 'DataSet', [args.changed]).then(function(result) {
            var aux = result;
        });
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('engine', engine);
});

/*global Configuration*/
/*global Util*/
/*global setLocalVar*/
/*global getLocalVar*/
/*global Config*/
/*global moment*/
